package search;

import java.util.ArrayList;
import java.util.Iterator;

import eightPuzzle.EightPuzzleState;

/** An ordered list of search nodes that have not yet been expanded.
 * @author Jeff Heflin 
 */
public class FrontierQueue {
  
  // Although one might think a LinkedList would be better, since inserts
  // and deletes could occur anywhere in the queue, empirical tests have
  // shown that ArrayList is faster. Perhaps this is due to the ability
  // to efficiently locate a node at a specific location.
  ArrayList queue;
  
  public FrontierQueue() {
    queue = new ArrayList();
  }
  
  /** Insert a new node into the queue in order of f-cost */ 
  public void insert(SearchNode newNode) {
    
    int i = 0;
    boolean done = false;
    SearchNode qNode;
    
    while (i < queue.size() && !done) {
      qNode = (SearchNode)(queue.get(i));
      if (newNode.getEvalFunCost() < qNode.getEvalFunCost()) {
        queue.add(i, newNode);
        done = true;
      }
      else
        i++;
    }
    if (done == false)
      queue.add(newNode);
  }
  
  /** Return true if there are no nodes in the queue. */
  public boolean isEmpty() {
    return queue.isEmpty();
  }
  
  /** Remove the first node from the queue and return it. */
  public SearchNode removeFirst() {
    
    SearchNode temp = (SearchNode)(queue.get(0));
    queue.remove(0);
    return temp;
  }
  
  /** Display debugging information about the queue to standard output. */
  public void displayQueue() {
    
    System.out.print("{");
    Iterator nodes = queue.iterator();
    while (nodes.hasNext()) {
      SearchNode next = (SearchNode)nodes.next();
      System.out.print(next.getEvalFunCost() + ",");
//      System.out.print(((State)next.getState()).toString() + ", ");
    }
    System.out.println("}");
  }
  
}
